--[[
## Title: GridStatusAbsorbshields
## Notes: Grid status showing the remaining value left on absorption buffs and debuffs.
## Author: Eldrikt
## Special thanks to Johnny C. Lam and his addon GridStatusAbsrobs
]]--

local L = LibStub("AceLocale-3.0"):GetLocale("GridStatusAbsorbshields")
local GridRoster = Grid:GetModule("GridRoster")
local GridStatusAbsorbshields = Grid:NewStatusModule("GridStatusAbsorbshields")

GridStatusAbsorbshields.menuName = L["Absorbshields"]

local absorbShieldStatus = "AbsorbShieldStatus"



GridStatusAbsorbshields.defaultDB = {
	[absorbShieldStatus] = {
		text = L["Absorb Shields"],
		enable = true,
		color = { r = 1.0, g = 1.0, b = 0.0, a = 0.8 },
		priority = 90,
		range = false,
	},
}

local absorbDamageOptions = {
	
} 

function GridStatusAbsorbshields:PostInitialize()
	self:RegisterStatus(absorbShieldStatus, L["Absorbshields"], absorbDamageOptions,true)
	
	-- create a hidden tooltip
	CreateFrame( "GameTooltip", "GSAS_Scantip", nil, "GameTooltipTemplate" ); 
	GSAS_Scantip:SetOwner(UIParent, "ANCHOR_NONE")
	GSAS_Scantip:AddFontStrings(
					GSAS_Scantip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
					GSAS_Scantip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" ) );
	
end

function GridStatusAbsorbshields:PostReset()
	self:UpdateAllUnitAbsorbs()
end

function GridStatusAbsorbshields:EnabledStatusCount()
	local enable_count = 0
	for status, settings in pairs(self.db.profile) do
		if type(settings) == "table" and settings.enable then
			enable_count = enable_count + 1
		end
	end
	return enable_count
end

function GridStatusAbsorbshields:OnStatusEnable(status)
	self:RegisterMessage("Grid_UnitJoined", "ScanUnitAuras")
	self:RegisterEvent("UNIT_AURA", "ScanUnitAuras")
	self:RegisterEvent("UNIT_MAXHEALTH", "UpdateMaxHealth")
	self:UpdateAllUnitAbsorbs()
end

function GridStatusAbsorbshields:OnStatusDisable(status)
	self.core:SendStatusLostAllUnits(status)
	if self:EnabledStatusCount() == 0 then
		self:UnregisterMessage("Grid_UnitJoined")
		self:UnregisterEvent("UNIT_AURA")
		self:UnregisterEvent("UNIT_MAXHEALTH")
	end
end

function GridStatusAbsorbshields:UpdateAllUnitAbsorbs()
	self:UpdateMaxHealth("UpdateAllUnitAbsorbs", "player")
	for guid, unitid in GridRoster:IterateRoster() do
		self.ClearUnit(guid)
		self:ScanUnitAuras("UpdateAllUnitAbsorbs", unitid)
	end
end

local maxHealth = 0
local activeAbsorbs = {}
--[[
local zeroMT = { __index = function(self, key) return rawget(self, key) or 0 end }
do
	for _, v in pairs(activeAbsorbs) do
		setmetatable(v, zeroMT)
	end
end
]]--

function GridStatusAbsorbshields.ClearUnit(guid)
	if not guid then return end
	for _, v in pairs(activeAbsorbs) do
		v[guid] = nil
	end
end

local function HumanizeNumber(number)
	if number < 1000 then
		return string.format("%.0f", number)
	elseif number < 100000 then
		return string.format("%.1fk", number/1000)
	elseif number < 1000000 then
		return string.format("%.0fk", number/1000)
	elseif number < 100000000 then
		return string.format("%.1fm", number/1000000)
	else
		return string.format("%.0fm", number/1000000)
	end
end

function GridStatusAbsorbshields:UpdateAbsorbDamageStatus(guid, unit)
	while true do
		local settings = self.db.profile[absorbShieldStatus]
		if not settings.enable then break end

		if not (maxHealth > 0) then break end

		local absorb = activeAbsorbs[guid] 
		if ( UnitHealth(unit) < maxHealth) then break end
		
		--local msg = absorb 
		--if msg == nil then msg = "nil" end
		--DEFAULT_CHAT_FRAME:AddMessage("v="..absorb..",guid="..guid..",unit="..unit)
		--DEFAULT_CHAT_FRAME:AddMessage(msg)
		
		if absorb > 0 then
			local text = HumanizeNumber(absorb)
			self.core:SendStatusGained(guid, absorbShieldStatus, settings.priority, nil, settings.color, text)
		else
			break
		end
		
		return
	end
	self.core:SendStatusLost(guid, absorbShieldStatus)
end

local playerGUID = UnitGUID("player")
function GridStatusAbsorbshields:UpdateMaxHealth(event, unit)
	self:Debug("UpdateMaxHealth", event, unit)
	local guid = UnitGUID(unit)
	if guid == playerGUID then
		maxHealth = UnitHealthMax(unit) or 0
	end
end

function GridStatusAbsorbshields:ScanUnitAuras(event, unit)
	local guid = UnitGUID(unit)
	if not GridRoster:IsGUIDInRaid(guid) then
		self.ClearUnit(guid)
		return
	end

	self:Debug("ScanUnitAuras", event, unit, guid)
		
	local function ScanUnitAbsorbs(unit, filter)
		local absorbs = 0

		local i = 1
		
		local name = UnitAura(unit, i, filter)
		
		while name do
			GSAS_Scantip:ClearLines()
			GSAS_Scantip:SetUnitBuff(unit,name)
			local textline = GSAS_ScantipTextLeft2:GetText();
			if textline then
				local absrb = strmatch(textline, "Absorb%a+ %d+")
				if absrb then
					local absrbval = strmatch(absrb,"%d+")
					absorbs = absorbs + absrbval --["any"]["any"]
				end
			end
			i = i + 1
			name = UnitAura(unit, i, filter)
		end
		return absorbs
	end --ScanUnitAbsorbs
	
	-- Only perform the expensive scan of buffs if the damage absorption status is enabled.
	if self.db.profile[absorbShieldStatus].enable then
		local absorbs = ScanUnitAbsorbs(unit, "HELPFUL")

		-- Update damage absorption status.
		local update = false
		if activeAbsorbs[guid] ~= absorbs then --[k]
			activeAbsorbs[guid] = absorbs --[k]
			--DEFAULT_CHAT_FRAME:AddMessage("v="..v..",guid="..guid..",unit="..unit)
			update = true
		end

		if update then
			self:UpdateAbsorbDamageStatus(guid, unit)
		end
	end
end